//= require <dSS/SaveSceneDialog>

Ext.namespace('dSS', 'dSA');

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function() {

        // this controls the single click expand option
        this.annoyingMode = true;
        this.activeNode = null;

        this.dirtyCount = 0;
        this.dirtyDevices = {};

        this.treePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            ctCls: 'x-panel-no-top-border',
            root: new Ext.tree.AsyncTreeNode(),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        this.apartmentTreePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            rootVisible: false,
            border: false,
            bodyBorder: false,
            root: new Ext.tree.AsyncTreeNode({
                children: this.genererateApartmentNodes(),
                id: 'root',
                singleClickExpand: this.annoyingMode,
                leaf: false
            }),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.menuPanel = new Ext.Panel({
            title: dSS._("Scene", "Scenes", 2),
            region: 'west',
            layout: 'border',
            split: true,
            width: 225,
            minSize: 175,
            tbar: this.buildTopToolbar(),
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'panel',
                    ref: 'zoneMenuArea',
                    layout: 'fit',
                    region: 'center',
                    split: true,
                    items:
                    [
                        this.treePanel
                    ]
                },
                {
                    xtype: 'panel',
                    ref: 'apartmentMenuArea',
                    layout: 'fit',
                    region: 'south',
                    //height: 80,
                    split: true,
                    items:
                    [
                        this.apartmentTreePanel
                    ]
                }
            ]
        });

        this.saveText = new Ext.form.Label({
            text: dSS._("The current settings can be saved for the selected " +
                        "scene."),
            colspan: 3
        });

        this.discardButton = new Ext.Button({
            text: dSS._("Discard"),
            disabled: true,
            minWidth: 100,
            handler: this.processSelectedScene.createDelegate(this,[false])
        });

        this.saveButton = new Ext.Button({
            text: dSS._("Save"),
            minWidth: 100,
            handler: this.saveSelectedScene.createDelegate(this)
        });

        this.saveToAnotherSceneButton = new Ext.Button({
            text: dSS._("Save to another scene"),
            handler: this.saveToAnotherScene.createDelegate(this),
            scope: this,
            colspan: 3,
            width: '100%'
        });

        var savePanel = new Ext.Panel({
            layout: 'table',
            border: false,
            scope: this,
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth')-58,
            layoutConfig: {
                columns: 3,
                tableAttrs: {
                    width: '100%'
                }
            },
            ref: 'saveScene',
            items:
            [
                this.saveText,
                {
                    xtype: 'box',
                    height: 10,
                    colspan: 3
                },
                {
                    xtype: 'panel',
                    layout: 'hbox',
                    border: false,
                    height: 30,
                    items:
                    [
                        this.saveButton,
                        {
                            xtype: 'box',
                            flex: 3
                        },
                        this.discardButton
                    ]
                },
                {
                    xtype: 'box',
                    height: 10,
                    colspan: 3
                },
                this.saveToAnotherSceneButton
            ]
        });

        this.controlPanel = new Ext.Panel({
            region: 'east',
            ref: 'controlPanel',
            disabled: true,
            containerScroll: true,
            bodyCssClass: 'x-panel-no-bottom-border',
            autoScroll: true,
            tbar:
            {
                height: 26,
                items:
                [
                    {
                        xtype: 'tbtext',
                        text: '&nbsp;',
                        width: 17,
                        ref: 'icon'
                    },
                    {
                        xtype: 'tbtext',
                        cls: 'x-black-text x-text-align-left x-text-cut',
                        ref: 'message'
                    }
                ],
                cls: 'x-panel-no-top-border'
            },
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth')-20,
            items:
            [
                {
                    xtype: 'dssdevicesettings',
                    border: false,
                    ref: 'settings',
                    embedded: true,
                    devices: [],
                    index: -1
                },
                {
                    xtype: 'panel',
                    padding: 10,
                    //border: false,
                    bodyCssClass: 'x-panel-no-right-border ' +
                                  'x-panel-no-left-border ' +
                                  'x-panel-no-bottom-border',
                    items: [ savePanel ]
                }
            ]
        });

        var contentPanel = {
            ref: 'contentPanel',
            region: 'center',
            title: dSS._("Device", "Devices", 2),
            bodyCssClass: 'x-panel-no-right-border',
            layout: 'border',
            items: [
                {
                    xtype: 'dssdevicepanel',
                    ref: 'devicePanel',
                    border: false,
                    frame: false,
                    region: 'center',
                    featureConfig: dSS.config.FeatureConfig.SCENE
                },
                this.controlPanel
            ]
        };

        Ext.apply(this, {
            layout: 'border',
            border: false,
            items: [ this.menuPanel, contentPanel ]
        });

        this.addEvents('treeredrawcomplte');
        this.dirtyTree = false;
        this.updateTree();


        dSS.data.Loader.getZoneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getZoneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
                if (dSS.config.firstLoad === true) {
                    var fn =
                        dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
                    fn();
                    dSS.data.Loader.load();
                    dSS.config.firstLoad = false;
                }

                this.controlPanel.settings.setRowSelectionModel(
                    this.contentPanel.devicePanel.getSelectionModel());

                this.contentPanel.devicePanel.on('rowclick', function(grid) {
                        var sel = grid.getSelectionModel();
                        var rec = sel.getSelected();
                        this.controlSaveButton(rec);
                    },
                    this);

                this.contentPanel.devicePanel.on('sortchange', function(grid) {
                        var devices = grid.getStore().getRange();
                        if (devices.length > 0) {
                            this.controlPanel.settings.loadDevices(devices);
                        }
                    },
                    this);

                this.contentPanel.devicePanel.getSelectionModel().on(
                        'selectionchange', function(sel) {
                            var rec = sel.getSelected();
                            this.controlSaveButton(rec);
                        },
                        this);
            },
            this);
        this.on('activate', function() {
            if (this.activeNode != null) {
                this.activeNode.setCls();
                this.activeNode = null;
            }
            this.handleTreeView();

            if (this.menuSizeSet !== true) {
                var total = this.menuPanel.ownerCt.getHeight();
                this.menuPanel.apartmentMenuArea.setHeight(total / 3);
                this.menuPanel.zoneMenuArea.setHeight((total / 3)*2);
                this.menuPanel.syncSize();
                this.menuSizeSet = true;
            }
        },
        this);

        this.on('deactivate', function() {
                this.dirtyCount = 0;
                this.dirtyDevices = {};
            },
            this);

        this.controlPanel.settings.on('valuechanged', function(dsuid) {
                if (this.dirtyDevices[dsuid] !== true) {
                    this.dirtyCount++;
                    this.dirtyDevices[dsuid] = true;
                }
                if (this.dirtyCount > 0) {
                    this.discardButton.enable();
                }
                var node = this.getSelectedNode();
                if (!node) {
                    return;
                }
                var ids = this.getZoneSceneGroup(node.id);
                var data = this.getNodeDisplayData(ids.zoneId, ids.groupId,
                                                   ids.sceneNumber);
                this.setSaveText(data.name, this.dirtyCount);
                if (node.isLeaf()) {
                    node.setCls();
                }
                if (this.activeNode != null) {
                    this.activeNode.setCls();
                    this.activeNode = null;
                }
            },
            this);

        this.controlPanel.settings.on('updatetitleicon', function(icon) {
                var tbar = this.controlPanel.getTopToolbar();
                if (icon === 'loading-indicator-small') {
                    tbar.message.setText(dSS._("Loading settings") + '...');
                    tbar.icon.addClass('loading-indicator loading-indicator-small');
                } else {
                    tbar.message.setText("&nbsp;");
                    tbar.icon.removeClass('loading-indicator');
                    tbar.icon.removeClass('loading-indicator-small');
                }
            },
            this);

        this.on('redrawtree', this.handleTreeView.createDelegate(this), this,
                { buffer: 1000 });

        dSS.MessageBus.on('scenesmarkdirty',
                           this.markDirty.createDelegate(this), this);

        dSS.MessageBus.on('reloadapartment',
            dSS.util.maskViewport.createDelegate(this, [ 'mask' ]),
            this);

        dSS.MessageBus.on('structureloaded',
            dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]),
            this);

        dSS.MessageBus.on('reloaddevicescenevalue', function() {
                var sm = this.contentPanel.devicePanel.getSelectionModel();
                if (sm.hasSelection()) {
                    this.controlPanel.settings.updateView(false);
                } else {
                    this.controlPanbel.settings.clearDialog();
                }
            }, this);

        this.treePanel.getSelectionModel().on('beforeselect', function(sel, node, oldnode) {
                if (node.isLeaf() && (node.disabled === true)) {
                    return false;
                }
                this.apartmentTreePanel.getSelectionModel().clearSelections();
            },
            this);

        this.treePanel.on('contextmenu', this.handleContextMenu, this);
        this.apartmentTreePanel.on('contextmenu', this.handleContextMenu, this);

        this.apartmentTreePanel.getSelectionModel().on('beforeselect',
                function(tree, node) {
                    this.treePanel.getSelectionModel().clearSelections();
                },
                this);

    },
    controlSaveButton: function(rec) {
        this.saveButton.enable();
        this.saveToAnotherSceneButton.enable();
        if (rec) {
            var arr = rec.get('groups');
            if (arr.length > 0) {
                if (arr[0] === dSS.config.groups.TEMPCONTROL) {
                    this.saveButton.disable();
                    this.saveToAnotherSceneButton.disable();
                }
            }
        }
    },
    setSaveText: function(sceneName, dirtyDevices) {
        if (dirtyDevices > 0) {
            this.saveText.update(dSS.gt.strargs(
                dSS._("The values for scene %1 were modified on %2 " +
                      "device and have not yet been saved. What would " +
                      "you like to do with current settings?",
                      "The values for scene %1 were modified on %2 " +
                      "devices and have not yet been saved. What would " +
                      "you like to do with current settings?", dirtyDevices),
                [
                    '&quot;' + sceneName + '&quot;', dirtyDevices
                ]));
        } else {
            this.saveText.update(dSS.gt.strargs(
                dSS._("The current settings can be saved for the selected " +
                      "scene (%1)."),
                [
                    sceneName
                ]));
        }
    },
    handleContextMenu: function(node, event) {
            event.preventDefault();
            event.stopEvent();

            if (!node.isSelected()) {
                node.select();
                this.handleNodeClick(node);
            }

            if (!node.isLeaf() || (node.disabled === true)) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: dSS._("Call scene"),
                iconCls: 'callSceneIcon',
                handler: this.processScene.createDelegate(this, [node, false])
            });
            this.contextMenu.add(menuItem);

            if (node.attributes.undo === true) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Undo scene"),
                    iconCls: 'undoSceneIcon',
                    handler: this.processScene.createDelegate(this, [node, true])
                });
                this.contextMenu.add(menuItem);
            }

            var path = node.getPath().split('/').slice(1);
            if (path.length === 4) {
                var ids = this.getZoneSceneGroup(node.id);
                if (ids.groupId != null) {
                    this.contextMenu.add('-');
                    menuItem = new Ext.menu.Item({
                        text: dSS._("Rename scene"),
                        iconCls: 'editIcon',
                        handler: this.renameSelectedScene.createDelegate(this,
                                                                         [node])
                    });
                    this.contextMenu.add(menuItem);
                }
            }

            var zoneId = parseInt(path[1], 10);
            if (zoneId === 0) {
                this.contextMenu.add('-');
                menuItem = new Ext.menu.Item({
                    text: dSS._("Save scene"),
                    iconCls: 'saveScene',
                    handler: this.saveSelectedScene.createDelegate(this,
                                                                     [node])
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
    markDirty: function() {
        this.dirtyTree = true;
        if (this.isVisible() === true) {
            this.fireEvent('redrawtree');
        }
    },

    handleTreeView: function() {
        if (this.dirtyTree === true) {
            this.updateTree();
            return;
        }

        var selModel = this.treePanel.getSelectionModel();
        var node = selModel.getSelectedNode();
        var root = this.treePanel.getRootNode();

        if (node == null) {
            selModel = this.apartmentTreePanel.getSelectionModel();
            node = selModel.getSelectedNode();
        }
        if (node == null) {
            if (root.hasChildNodes()) {
                root.firstChild.select();
                this.handleNodeClick(root.firstChild);
            } else {
              this.updateTree();
            }
        } else {
            this.handleNodeClick(node);
        }
    },

    getSceneName: function(zone, group, scene) {
        var sceneStore = dSS.data.Loader.getSceneStore();
        var rec = sceneStore.getById(zone + '-' + group + '-' + scene);
        if (rec === undefined) {
            return '';
        }

        var name = rec.get('name');
        if ((name === undefined) || (name.length === 0)) {
            return '';
        }

        return ' - ' + name;
    },

    getNodeDisplayData: function(zone, group, scene) {
        var name = dSS.config.sceneNames[scene];
        var tip = dSS.config.sceneTips[scene];
        if (group === dSS.config.groups.SHADOW) {
            switch (scene) {
                case dSS.config.scenes.T0_S0:
                    name = "Down";
                    tip = dSS.config.sceneTipsShadowOverrides[scene];
                    break;
                case dSS.config.scenes.T0_S1:
                    name = "Up";
                    tip = dSS.config.sceneTipsShadowOverrides[scene];
                    break;
                case dSS.config.scenes.T1_S1:
                    name = "Area 1 up";
                    break;
                case dSS.config.scenes.T2_S1:
                    name = "Area 2 up";
                    break;
                case dSS.config.scenes.T3_S1:
                    name = "Area 3 up";
                    break;
                case dSS.config.scenes.T4_S1:
                    name = "Area 4 up";
                    break;
                default:
                    break;
            }
        }

        var text = dSS._(name) + this.getSceneName(zone, group, scene);
        return { name: text, tip: dSS._(tip) };
    },

    generateSceneNode: function(zone, group, scene, icon, locks) {
        var node = {};
        var data = this.getNodeDisplayData(zone, group, scene);
        var name = data.name;
        var tip = data.tip;

        var locked = false;
        if (locks.indexOf(scene) !== -1) {
            locked = true;
        }

        node.text = name;
        node.id = zone + '-' + group + '-' + scene;
        node.leaf = true;
        node.iconCls = icon;
        node.qtip = tip;
        node.lockedScene = locked;
        return node;
    },

    // generates scenes 0-4 for a given zone/group
    generate0to4Scenes: function(zone, group, icon, locks) {
        var addScenes = [ dSS.config.scenes.T0_S0 ];

        if (group === dSS.config.groups.LIGHT) {
            addScenes.push(dSS.config.scenes.AUTO_OFF);
        }

        addScenes = addScenes.concat(
        [
            dSS.config.scenes.T0_S1, dSS.config.scenes.T0_S2,
            dSS.config.scenes.T0_S3, dSS.config.scenes.T0_S4
        ]);

        if (group === dSS.config.groups.SHADOW) {
            addScenes.push(dSS.config.scenes.SUNPROTECTION);
        }

        var scenes = [];

        var i;
        for (i = 0; i < addScenes.length; i++) {
            var nodeIcon = icon;
            // off scenes have special icons
            if (addScenes[i] === dSS.config.scenes.T0_S0) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            } else if (addScenes[i] === dSS.config.scenes.AUTO_OFF) {
                nodeIcon = 'autoStandby';
            } else if (addScenes[i] === dSS.config.scenes.SUNPROTECTION) {
                nodeIcon = 'sunprotection';
            }
            scenes.push(
                this.generateSceneNode(zone, group, addScenes[i], nodeIcon, locks));
        }
        return scenes;
    },

    generatePushbuttonScenes: function(zone, group, icon, cfg, locks) {
        var scenes = [];
        var sceneNumbers = [];

        if (this.disableSceneFilter === true) {
            cfg =
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS.config.deviceParams.PUSHBUTTON_GROUP8
            ];

        } else {
            cfg = dSS.util.UniqueArray(cfg, true);
            if (cfg.length < 1) {
                return scenes;
            }
        }

        var offScenes =
        [
            dSS.config.scenes.EX5, dSS.config.scenes.EX10,
            dSS.config.scenes.EX15, dSS.config.scenes.EX20
        ];

        var lastScenes =
        [
            dSS.config.scenes.T1_S4, dSS.config.scenes.T2_S4,
            dSS.config.scenes.T3_S4, dSS.config.scenes.T4_S1
        ];

        var i;
        for (i = 0; i < cfg.length; i++) {
            switch (cfg[i]) {
                case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                    // device pushbutton - no extra nodes
                case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                    // zone pushbutton, defaults scenes 0-4, no extra nodes
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
                    // mood area 1
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T1_S0);
                        sceneNumbers.push(dSS.config.scenes.T1_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T1_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
                    // mood area 2
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T2_S0);
                        sceneNumbers.push(dSS.config.scenes.T2_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T2_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
                    // mood area 3
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T3_S0);
                        sceneNumbers.push(dSS.config.scenes.T3_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T3_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
                    // mood area 4
                    if (this.disableSceneFilter === true) {
                        offScenes.push(dSS.config.scenes.T4_S0);
                        sceneNumbers.push(dSS.config.scenes.T4_S0);
                    }
                    sceneNumbers.push(dSS.config.scenes.T4_S1);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                    // moods 5-9
                    sceneNumbers.push(dSS.config.scenes.EX5);
                    sceneNumbers.push(dSS.config.scenes.EX6);
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                    // moods 7-9
                    sceneNumbers =
                        [ dSS.config.scenes.T1_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T1_S2);
                    sceneNumbers.push(dSS.config.scenes.T1_S3);
                    sceneNumbers.push(dSS.config.scenes.T1_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                    // moods 10-14
                    sceneNumbers.push(dSS.config.scenes.EX10);
                    sceneNumbers.push(dSS.config.scenes.EX11);
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                    // moods 12-14
                    sceneNumbers =
                        [ dSS.config.scenes.T2_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T2_S2);
                    sceneNumbers.push(dSS.config.scenes.T2_S3);
                    sceneNumbers.push(dSS.config.scenes.T2_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                    // moods 15-19
                    sceneNumbers.push(dSS.config.scenes.EX15);
                    sceneNumbers.push(dSS.config.scenes.EX16);
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                    // moods 17-19
                    sceneNumbers =
                        [ dSS.config.scenes.T3_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T3_S2);
                    sceneNumbers.push(dSS.config.scenes.T3_S3);
                    sceneNumbers.push(dSS.config.scenes.T3_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                    // moods 20-24
                    sceneNumbers.push(dSS.config.scenes.EX20);
                    sceneNumbers.push(dSS.config.scenes.EX21);
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                    // moods 22-24
                    sceneNumbers =
                        [ dSS.config.scenes.T4_S1 ].concat(sceneNumbers);
                    sceneNumbers.push(dSS.config.scenes.T4_S2);
                    sceneNumbers.push(dSS.config.scenes.T4_S3);
                    sceneNumbers.push(dSS.config.scenes.T4_S4);
                    break;
                default:
                    break;
            }
        }

        sceneNumbers = dSS.util.UniqueArray(sceneNumbers, true);
        sceneNumbers.sort(function(a, b) {
            return dSS.config.sceneSortOrder[a] - dSS.config.sceneSortOrder[b];
        });

        var len = sceneNumbers.length;

        for (i = 0; i < sceneNumbers.length; i++) {
            var nodeIcon = icon;
            if (offScenes.indexOf(sceneNumbers[i]) !== -1) {
                if (icon.substring(icon.length - 2) === 'On') {
                    nodeIcon = icon.substring(0, icon.length - 2) + 'Off';
                } else {
                    nodeIcon = icon + 'Off';
                }
            }

            var prependSeparator = false;
            var appendSeparator = false;

            switch (sceneNumbers[i]) {
                case dSS.config.scenes.T1_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX6) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T2_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX11) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T3_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX16) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.T4_S2:
                    if (sceneNumbers[i - 1] !== dSS.config.scenes.EX21) {
                        prependSeparator = true;
                    }
                    break;
                case dSS.config.scenes.EX5:
                case dSS.config.scenes.EX10:
                case dSS.config.scenes.EX15:
                case dSS.config.scenes.EX20:
                        prependSeparator = true;
                    break;

                default:
                    break;
            }

            if (prependSeparator) {
                scenes.push(this.generateSeparatorNode());
            }
            scenes.push(
                    this.generateSceneNode(zone, group, sceneNumbers[i],
                    nodeIcon, locks));

        }
        return scenes;
    },

    getLockedScenes: function(clusterLocks, group) {
        var lockedScenes = [];
        Ext.each(clusterLocks, function(lock) {
            if (lock.deviceClass == group) {
                lockedScenes = lock.lockedScenes;
                return false;
            }
            return true;
        });

        return lockedScenes;
    },
    // for leafs the id is encoded as: zone id - group - scene number
    // when no group is desired, 0 is used
    generateSubNodes: function(zone, groups, clusterLocks) {
        var zoneId = zone.get('id');
        var retNodes = [];
        var childNodes = [];
        if (groups.LIGHT === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.LIGHT);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.LIGHT,
                                                 'lightbulbOn',
                                                 locks);
            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.LIGHT,
                                           'lightbulbOn',
                                           groups.LIGHTpb,
                                           locks));
            retNodes.push(
            {
                text: dSS._("Light scenes"),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT],
                children: childNodes
            });
        }

        if (groups.SHADOW === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.SHADOW);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.SHADOW,
                                                 'shadesIcon',
                                                 locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.SHADOW,
                                           'shadesIcon',
                                           groups.SHADOWpb, locks));

            retNodes.push({
                text: dSS._("Shade scenes"),
                id: zoneId + '-' + dSS.config.groups.SHADOW + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW],
                children: childNodes
            });
        }

        if (groups.HEATING === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.HEATING);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.HEATING,
                                                 'climateIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.HEATING,
                                           'climateIcon',
                                           groups.HEATINGpb, locks));

            retNodes.push({
                text: dSS._("Heating scenes"),
                id: zoneId + '-' + dSS.config.groups.HEATING + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.HEATING],
                children: childNodes
            });
        }

        if (groups.COOLING === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.COOLING);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.COOLING,
                                                 'climateIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.COOLING,
                                           'climateIcon',
                                           groups.COOLINGpb, locks));

            retNodes.push({
              text: dSS._("Cooling scenes"),
              id: zoneId + '-' + dSS.config.groups.COOLING + '-all',
              leaf: false,
              singleClickExpand: this.annoyingMode,
              iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.HEATING],
              children: childNodes
            });
        }

        if (groups.VENTILATION === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.VENTILATION);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.VENTILATION,
                                                 'climateIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.VENTILATION,
                                           'climateIcon',
                                           groups.VENTILATIONpb, locks));

            retNodes.push({
              text: dSS._("Ventilation scenes"),
              id: zoneId + '-' + dSS.config.groups.VENTILATION + '-all',
              leaf: false,
              singleClickExpand: this.annoyingMode,
              iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.HEATING],
              children: childNodes
            });
        }

        if (groups.AUDIO === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.AUDIO);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.AUDIO,
                                                 'audioIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.AUDIO,
                                           'audioIcon',
                                           groups.AUDIOpb, locks));

            retNodes.push({
                text: dSS._("Audio scenes"),
                id: zoneId + '-' + dSS.config.groups.AUDIO + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO],
                children: childNodes
            });
        }

        if (groups.VIDEO === true) {
            var locks = this.getLockedScenes(clusterLocks, dSS.config.groups.VIDEO);
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.VIDEO,
                                                 'videoIcon', locks);

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.VIDEO,
                                           'videoIcon',
                                           groups.VIDEOpb, locks));

            retNodes.push({
                text: dSS._("Video scenes"),
                id: zoneId + '-' + dSS.config.groups.VIDEO + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                iconCls: dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO],
                children: childNodes
            });
        }

        if (groups.WHITE === true) {
            childNodes = this.generate0to4Scenes(zoneId,
                                                 dSS.config.groups.WHITE,
                                                 'whiteIcon');

            childNodes = childNodes.concat(this.generatePushbuttonScenes(
                                           zoneId,
                                           dSS.config.groups.WHITE,
                                           'whiteIcon',
                                           groups.WHITEpb));

            retNodes.push({
                text: dSS._("White goods scenes"),
                id: zoneId + '-' + dSS.config.groups.WHITE + '-all',
                leaf: false,
                singleClickExpand: this.annoyingMode,
                children: childNodes
            });
        }

        var ls = zone.get('lockedApartmentScenes');
        retNodes = retNodes.concat(
            [
                {
                    text: dSS._("Standby"),
                    id: zoneId + '-all-' + dSS.config.scenes.STANDBY,
                    leaf: true,
                    iconCls: 'standByIcon',
                    lockedScene: (ls.indexOf(dSS.config.scenes.STANDBY) !== -1)
                },
                {
                    text: dSS._("Deep off"),
                    id: zoneId + '-all-' + dSS.config.scenes.DEEP_OFF,
                    leaf: true,
                    iconCls: 'deepOffIcon',
                    lockedScene: (ls.indexOf(dSS.config.scenes.DEEP_OFF) !== -1)
                },
                {
                    text: dSS._("Sleeping"),
                    id: zoneId + '-all-' + dSS.config.scenes.SLEEPING,
                    leaf: true,
                    iconCls: 'sleepingIcon',
                    lockedScene: (ls.indexOf(dSS.config.scenes.SLEEPING) !== -1)
                },
                {
                    text: dSS._("Wake up"),
                    id: zoneId + '-all-' + dSS.config.scenes.WAKE_UP,
                    leaf: true,
                    iconCls: 'wakeupIcon',
                    lockedScene: (ls.indexOf(dSS.config.scenes.WAKE_UP) !== -1)
                }
            ]
        );

        return retNodes;
    },

    genererateApartmentNodes: function() {
        return [
            {
                text: dSS._("Leave home"),
                id: '0' + '-all-' + dSS.config.scenes.ABSENT,
                leaf: true,
                iconCls: 'leaveHomeIcon'
            },
            {
                text: dSS._("Coming home"),
                id: '0' + '-all-' + dSS.config.scenes.PRESENT,
                leaf: true,
                iconCls: 'comeHomeIcon'
            },
            {
                text: dSS._("Door bell"),
                id: '0' + '-all-' + dSS.config.scenes.SIG_BELL,
                leaf: true,
                iconCls: 'doorBellIcon'
            },
            this.generateSeparatorNode(),
            {
                text: dSS._("Rain"),
                id: '0' + '-all-' + dSS.config.scenes.RAIN,
                iconCls: 'rainIcon',
                undo: false,
                leaf: true
            },
            {
                text: dSS._("No rain"),
                id: '0' + '-all-' + dSS.config.scenes.NO_RAIN,
                iconCls: 'noRainIcon',
                undo: false,
                leaf: true
            },
            {
                text: dSS._("Wind"),
                id: '0' + '-all-' + dSS.config.scenes.WIND,
                iconCls: 'windIcon',
                undo: false,
                leaf: true
            },
            {
                text: dSS._("No wind"),
                id: '0' + '-all-' + dSS.config.scenes.NO_WIND,
                iconCls: 'noWindIcon',
                undo: false,
                leaf: true
            },
            {
                text: dSS._("Hail"),
                id: '0' + '-all-' + dSS.config.scenes.HAIL,
                iconCls: 'hailIcon',
                undo: false,
                leaf: true
            },
            {
                text: dSS._("No hail"),
                id: '0' + '-all-' + dSS.config.scenes.NO_HAIL,
                iconCls: 'noHailIcon',
                undo: false,
                leaf: true
            },
            this.generateSeparatorNode(),
            {
                text: dSS._("Panic"),
                id: '0' + '-all-' + dSS.config.scenes.PANIC,
                iconCls: 'panicIcon',
                undo: true,
                leaf: true
            },
            {
                text: dSS._("Fire"),
                id: '0' + '-all-' + dSS.config.scenes.FIRE,
                iconCls: 'fireIcon',
                undo: true,
                leaf: true
            },
            {
                text: dSS._("Alarm 1"),
                id: '0' + '-all-' + dSS.config.scenes.ALARM1,
                iconCls: 'alarm1Icon',
                undo: true,
                leaf: true
            },
            {
                text: dSS._("Alarm 2"),
                id: '0' + '-all-' + dSS.config.scenes.ALARM2,
                iconCls: 'alarm2Icon',
                undo: true,
                leaf: true
            },
            {
                text: dSS._("Alarm 3"),
                id: '0' + '-all-' + dSS.config.scenes.ALARM3,
                iconCls: 'alarm3Icon',
                undo: true,
                leaf: true
            },
            {
                text: dSS._("Alarm 4"),
                id: '0' + '-all-' + dSS.config.scenes.ALARM4,
                iconCls: 'alarm4Icon',
                undo: true,
                leaf: true
            }
        ];
    },

    generateSeparatorNode: function() {
        return {
            text: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;' +
                  '&mdash;&mdash;&mdash;&mdash;&mdash;',
            disabled: true,
            cls: 'x-tree-noicon x-item-disabled mousedefault',
            isTarget: false,
            allowDrag: false,
            allowDrop: false,
            isSeparator: true,
            leaf: true,
            listeners: {
                contextmenu: function() { return false; }
            }
        };
    },

    updateTree: function() {
        var devStore = dSS.data.Loader.getDeviceStore();
        devStore.clearFilter();

        var panel = this.apartmentTreePanel;
        var selModel = panel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = '';

        if (selectedNode == null) {
            panel = this.treePanel;
            selModel = panel.getSelectionModel();
            selectedNode = selModel.getSelectedNode();
        }

        if (selectedNode != null) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var nodes = [];
        zStore.each(
        function(record) {
            if ((record.get('id') !== 0) && (devStore.find('zoneId', record.get('id')) >= 0)) {
                var deviceTypesInZone = {
                    LIGHT:         false,
                    LIGHTpb:       [],
                    SHADOW:        false,
                    SHADOWpb:      [],
                    HEATING:       false,
                    HEATINGpb:     [],
                    COOLING:       false,
                    COOLINGpb:     [],
                    VENTILATION:   false,
                    VENTILATIONpb: [],
                    AUDIO:         false,
                    AUDIOpb:       [],
                    VIDEO:         false,
                    VIDEOpb:       [],
                    WHITE:         false,
                    WHITEpb:       []
                };
                var addZone = false;
                var pushbuttonConfigs = [];
                devStore.each(function(device) {
                    var pbConfig;
                    if (device.get('zoneId') === record.get('id')) {
                        if (device.get('isPresent') === false) {
                            return;
                        }

                        var buttonID = device.get('buttonID');
                        if ((buttonID !== 0xff) &&
                            ((device.get('productRevision') >= dSS.config.PushbuttonMinFW) ||
                             (device.get('isVdcDevice') === true))) {
                            pbConfig = (buttonID & 0x0000000f);
                        }

                        var fid = dSS.util.parseFunctionID(device.get('functionID'));
                        var joker;

                        switch (fid[0]) {
                            case dSS.config.groups.LIGHT:
                                deviceTypesInZone.LIGHT = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.LIGHTpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.SHADOW:
                                deviceTypesInZone.SHADOW = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.SHADOWpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.HEATING:
                                var groupMember = dSS.util.getHeatingGroup(device.get('groups'));

                                if ((groupMember === dSS.config.groups.COOLING) &&
                                    (device.get('outputMode') !== 
                                     dSS.config.deviceParams.OUTPUT_MODE_DISABLED)) {
                                    deviceTypesInZone.COOLING = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.COOLINGpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                }

                                if ((groupMember === dSS.config.groups.VENTILATION) &&
                                    (device.get('outputMode') !== 
                                     dSS.config.deviceParams.OUTPUT_MODE_DISABLED)) {
                                    deviceTypesInZone.VENTILATION = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.VENTILATIONpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                }

                                if ((groupMember === dSS.config.groups.HEATING) &&
                                    (device.get('outputMode') !== 
                                     dSS.config.deviceParams.OUTPUT_MODE_DISABLED)) {
                                    deviceTypesInZone.HEATING = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.HEATINGpb.push(pbConfig);
                                    }
                                    addZone = true;
                                    break;
                                }
                                break;
                            case dSS.config.groups.AUDIO:
                                deviceTypesInZone.AUDIO = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.AUDIOpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.VIDEO:
                                deviceTypesInZone.VIDEO = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.VIDEOpb.push(pbConfig);
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.JOKER:
                                deviceTypesInZone.JOKER = true;
                                joker = dSS.util.getJokerGroup(
                                                        device.get('groups'));
                                if (joker === 1) {
                                    deviceTypesInZone.LIGHT = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.LIGHTpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 2) {
                                    deviceTypesInZone.SHADOW = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.SHADOWpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 3) {
                                    deviceTypesInZone.HEATING = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.HEATINGpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 4) {
                                    deviceTypesInZone.AUDIO = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.AUDIOpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 5) {
                                    deviceTypesInZone.VIDEO = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.VIDEOpb.push(
                                                pbConfig);
                                    }
                                } else if (joker === 9) {
                                    deviceTypesInZone.WHITE = true;
                                    if (pbConfig !== undefined) {
                                        deviceTypesInZone.WHITEpb.push(
                                                pbConfig);
                                    }
                                }
                                addZone = true;
                                break;
                            case dSS.config.groups.WHITE:
                                deviceTypesInZone.WHITE = true;
                                if (pbConfig !== undefined) {
                                    deviceTypesInZone.WHITEpb.push(pbConfig);
                                }

                                addZone = true;
                                break;
                            default:
                                break;
                        }
                    }
                });
                if (addZone === true) {
                    nodes.push({
                        text: record.get('name'),
                        id: record.get('id'),
                        leaf: false,
                        singleClickExpand: this.annoyingMode,
                        children: this.generateSubNodes(record,
                                                        deviceTypesInZone,
                                                        record.get('clusterLocks'))
                    });
                }
            }
            return true;
        },
        this);

        var root = new Ext.tree.AsyncTreeNode({
            children: nodes,
            id: 'root',
            singleClickExpand: this.annoyingMode,
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length > 0) {
            var tp = this.treePanel;
            panel.selectPath(selectedPath, null, function(result, node) {
                // if a node is gone, find the parend and try to select the
                // first child but othewise stay on the same level in the tree
                if (result === false) {
                    var selected = false;
                    var nodes = selectedPath.split('/');
                    if (nodes.length >= 2) {
                        var parentId = nodes[nodes.length - 2];
                        var parentNode = tp.getNodeById(parentId);
                        if (parentNode && (parentNode.firstChild != null)) {
                            parentNode.firstChild.select();
                            selected = true;
                        }
                    }
                    if (selected === false) {
                        if (root.firstChild != null) {
                            root.firstChild.select();
                            selected = true;
                        }
                    }

                    if (selected === false) {
                        if ((this.apartmentTreePanel.getRootNode() != null) &&
                            (this.apartmentTreePanel.getRootNode().firstChild != null)) {
                            this.apartmentTreePanel.getRootNode().firstChild.select();
                            selected = true;
                        }
                    }
                }
            });
        } else {
            if (root.firstChild != null) {
                root.firstChild.select();
            } else {
                if ((this.apartmentTreePanel.getRootNode() != null) &&
                    (this.apartmentTreePanel.getRootNode().firstChild != null)) {
                    this.apartmentTreePanel.getRootNode().firstChild.select();
                }
            }
        }

        if (selModel.getSelectedNode() != null) {
            this.handleNodeClick(selModel.getSelectedNode());
        } else {
            selModel = this.apartmentTreePanel.getSelectionModel();
            if (selModel.getSelectedNode() != null) {
                this.handleNodeClick(selModel.getSelectedNode());
            }
        }

        if (this.activeNode != null) {
            var an = panel.getNodeById(this.activeNode.id);
            if (an == null) {
                an = this.apartmentTreePanel.getNodeById(this.activeNode.id);
            }
            this.activeNode = an;
            if (this.activeNode != null) {
                this.activeNode.setCls('active-scene-node');
            }
        }


        this.dirtyTree = false;
        this.fireEvent('treeredrawcomplte');
    },

    handleNodeClick: function(node) {
        if (!this.isVisible()) {
            return;
        }

        if (node == null) {
            return;
        }

        if (node.isLeaf() === true) {
            this.menuPanel.getTopToolbar().get('callSceneButton').enable();
            if (node.attributes.undo === true) {
                this.menuPanel.getTopToolbar().get('undoSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
            }
        } else {
            this.menuPanel.getTopToolbar().get('callSceneButton').disable();
            this.menuPanel.getTopToolbar().get('undoSceneButton').disable();
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        if (path.length === 4) {
            var ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId != null) {
                this.menuPanel.getTopToolbar().get('renameSceneButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            }
            dSS.MessageBus.fireEvent('sceneselected', parseInt(path[3].split('-')[2]));
        } else {
            this.menuPanel.getTopToolbar().get('renameSceneButton').disable();
            dSS.MessageBus.fireEvent('sceneselected', 0);
        }

        if (path.length > 2) {
            var ids = this.getZoneSceneGroup(node.id);
            if (ids.groupId !== null) {
                this.menuPanel.getTopToolbar().get('readAreaConfigButton').enable();
            } else {
                this.menuPanel.getTopToolbar().get('readAreaConfigButton').disable();
            }
        } else {
            this.menuPanel.getTopToolbar().get('readAreaConfigButton').disable();
        }


        if (path.length < 2) {
            return;
        }

        var zoneId = parseInt(path[1], 10);
        if (zoneId === 0) {
            this.controlPanel.hide();
            this.contentPanel.devicePanel.reconfigureColumns('apartment');
            this.doLayout();
        } else {
            this.controlPanel.show();
            this.contentPanel.devicePanel.reconfigureColumns('zone');
            this.doLayout();
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        var sm = this.contentPanel.devicePanel.getSelectionModel();
        var rec = sm.getSelected();
        var selectedDSUID = "";
        if (rec !== undefined) {
            selectedDSUID = rec.get('dsuid');
        }
        store.filterBy(function(record) {
            if (record.get('isPresent') === false) {
                return false;
            }

            if (zoneId === 0) {
                return true;
            }

            if (record.get('zoneId') === zoneId) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));

                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === dSS.config.groups.HEATING) {
                            if (dSS.util.getHeatingGroup(record.get('groups')) === 
                                group) {
                                return true;
                            } else {
                                return false;
                            }
                        }
                        if (fid[0] === group) {
                            return true;
                        }

                        if ((fid[0] === dSS.config.groups.JOKER) &&
                            (dSS.util.getJokerGroup(record.get('groups')) ===
                             group)) {
                            return true;
                        }

                        return false;
                    }
                } else {
                    return true;
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);

                if (isNaN(scene)) {
                    return false;
                }
            }
            return false;
        });
        
        var devices = store.getRange();
        if (zoneId !== 0) {
            if (devices.length > 0) {
                this.controlPanel.settings.loadDevices(devices);
            }
            if ((node.isLeaf() === true) && (node.attributes.lockedScene !== true)) {
                this.controlPanel.enable();
                var ids = this.getZoneSceneGroup(node.id);
                var data = this.getNodeDisplayData(ids.zoneId, ids.groupId,
                                                   ids.sceneNumber);
                this.dirtyCount = 0;
                this.dirtyDevices = {};
                this.setSaveText(data.name, this.dirtyCount);
                this.discardButton.disable();
            } else {
                this.controlPanel.disable();
            }
        }
        if ((node.isLeaf() === true) && (devices.length > 0)) {
            if ((devices[0].get('dsuid') !== selectedDSUID) || this.activeNode) {
                sm.selectRow(0, false, false);
                this.contentPanel.devicePanel.handleDeviceRowSelect();
            }
            this.controlPanel.settings.setupButtons();
        } else {
            this.contentPanel.devicePanel.getSelectionModel().clearSelections();
            this.controlPanel.settings.clearDialog();
        }
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'editIcon',
                handler: this.renameSelectedScene.createDelegate(this),
                tooltip: dSS._("Rename selected scene"),
                id: 'renameSceneButton',
                disabled: true,
                scope: this
            },
            '-',
            {
                iconCls: 'callSceneIcon',
                handler: this.processSelectedScene.createDelegate(this,[false]),
                id: 'callSceneButton',
                tooltip: dSS._("Call selected scene"),
                disabled: true,
                scope: this
            },
            {
                iconCls: 'undoSceneIcon',
                handler: this.processSelectedScene.createDelegate(this,[true]),
                id: 'undoSceneButton',
                tooltip: dSS._("Undo selected scene"),
                disabled: true,
                scope: this
            },
            {
                iconCls: 'readAreaConfigIcon',
                handler: this.readAreaConfig.createDelegate(this),
                id: 'readAreaConfigButton',
                tooltip: dSS._("Read area configuration of devices " +
                               "for selected zone and group"),
                disabled: true,
                scope: this
            },
            '->',
            {
                iconCls: 'showAllScenesIcon',
                handler: this.showAllScenes.createDelegate(this),
                tooltip: dSS._("Display all available scenes"),
                ref: 'showAllScenesButton',
                enableToggle: true,
                scope: this
            }
        ];
    },
    showAllScenes: function(button) {
        if (button.pressed === true) {
            this.disableSceneFilter = true;
        } else {
            this.disableSceneFilter = false;
        }
        // might get called from "save as window", sync button
        if (button.pressed !==
                this.menuPanel.getTopToolbar().showAllScenesButton.pessed) {
            this.menuPanel.getTopToolbar().showAllScenesButton.toggle(
                    button.pressed, false);
        }
        this.markDirty();
    },
    getZoneSceneGroup: function(id) {
        var ret = {
            "zoneId": null,
            "sceneNumber": null,
            "groupId": null
        };

        var ids = id.split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        if (isNaN(zoneId)) {
            return ret;
        }

        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    getSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node == null) || (!node.isLeaf())) {
            throw dSS._("Can not identify scene: no scene selected");
        }

        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId == null) || (ids.sceneNumber == null)) {
            throw dSS._("Can not identify scene: invalid data");
        }

        return ids.sceneNumber;
    },

    processSelectedScene: function(undo) {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node == null) || (!node.isLeaf())) {
            if (undo === true) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you would like to undo"));
            } else {
                Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you would like to call"));
            }
            return;
        }

        this.processScene(node, undo);
    },

    readAreaConfig: function() {
        if (dSA.module !== true) {
            Ext.Msg.alert(dSS._("Notice"),
                          dSS._("This feature requires presence of the " +
                                "digitalSTROM Assistant, the setup is only " +
                                "available on the dSS11 hardware"));
            return;
        }

        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Nothing selected"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId == null) || (ids.groupId == null)) {
            return;
        }

        dSA.Reactor.fireEvent('readareaconfig', ids.zoneId, ids.groupId,
                              dSS.data.Loader.getDeviceStore());
    },

    processScene: function(node, undo) {
        if ((node == null) || (!node.isLeaf())) {
            return;
        }

        if ((undo === true) && (node.attributes.undo !== true)) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Undo is not available for this scene"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId == null) || (ids.sceneNumber == null)) {
            return;
        }

        if (this.activeNode != null) {
            this.activeNode.setCls();
        }

        node.setCls('active-scene-node');
        this.activeNode = node;
        this.discardButton.disable();
        this.dirtyCount = 0;
        this.dirtyDevices = {};
        var data = this.getNodeDisplayData(ids.zoneId, ids.groupId,
                                           ids.sceneNumber);
        this.setSaveText(data.name, this.dirtyCount);
        if (undo === true) {
            dSS.util.undoSceneOnZone(ids.zoneId, ids.sceneNumber);
        } else {
            dSS.util.callSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId,
                                     true);
        }
        if (ids.zoneId !== 0) {
            var sm = this.contentPanel.devicePanel.getSelectionModel();
            if (sm.hasSelection()) {
                this.controlPanel.settings.updateView(false);
            } else {
                this.controlPanbel.settings.clearDialog();
            }
        }
    },
    getSelectedNode: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }
        return node;
    },
    saveSelectedScene: function() {
        var node = this.getSelectedNode();
        if ((node == null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("No scene selected"), dSS._("No scene selected, please select the scene that you would like to save"));
            return;
        }

        this.saveScene(node);
    },
    saveScene: function(node) {
        if ((node == null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId == null) || (ids.sceneNumber == null)) {
            return;
        }

        if (ids.sceneNumber !== 0) {
            if ((this.activeNode != null) && (node !== this.activeNode)) {
                this.warnSaveDifferentScene(node, this.activeNode);
                return;
            }
            if (this.activeNode != null) {
                this.activeNode.setCls();
            }
            node.setCls('active-scene-node');
            this.activeNode = node;
            var data = this.getNodeDisplayData(ids.zoneId, ids.groupId,
                                               ids.sceneNumber);
            this.dirtyCount = 0;
            this.dirtyDevices = {};
            this.setSaveText(data.name, this.dirtyCount);
            this.discardButton.disable();

            dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);

            if ((ids.sceneNumber >= 6) && (ids.sceneNumber <= 9)) {
                var store = dSS.data.Loader.getDeviceStore();
                store.each(function(record) {
                    record.set('areas', []);
                });
            }
        } else {
            this.warnSaveScene0(node);
        }
    },
    warnSaveDifferentScene: function(node, activeNode) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId == null) || (ids.sceneNumber == null)) {
            return;
        }

        var aIds = this.getZoneSceneGroup(activeNode.id);
        if ((aIds.zoneId == null) || (aIds.sceneNumber == null)) {
            return;
        }

        var calledName = dSS._(dSS.config.sceneNames[aIds.sceneNumber]) +
            this.getSceneName(aIds.zoneId, aIds.groupId, aIds.sceneNumber);

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("Called scene") + ': &quot;' + calledName +
                        '&quot;<br/>' +
                  dSS._("Saving scene") + ': &quot;' + savingName +
                        '&quot;<br/><br/>' +
                  dSS._("You are about to save a scene that is different " +
                        "from the called scene.") + '<br/>' +
                  dSS._("Are you sure that you want to overwrite the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    if (this.activeNode != null) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;
                    this.dirtyCount = 0;
                    this.dirtyDevices = {};
                    this.setSaveText(savingName, this.dirtyCount);
                    this.discardButton.disable();
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                    if ((ids.sceneNumber >= 6) && (ids.sceneNumber <= 9)) {
                        var store = dSS.data.Loader.getDeviceStore();
                        store.each(function(record) {
                            record.set('areas', []);
                        });
                    }
                }
            },
            this
        );
    },
    warnSaveScene0: function(node) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId == null) || (ids.sceneNumber == null)) {
            return;
        }

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("You are about to save scene 0 which represents " +
                        "the off scene. Note that you might not be able to " +
                        "switch off all devices in case some devices in the " +
                        "selected room are turned on right now.") + '<br/>' +
                  dSS._("Are you sure you want to save the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    if (this.activeNode != null) {
                        this.activeNode.setCls();
                    }
                    node.setCls('active-scene-node');
                    this.activeNode = node;
                    this.dirtyCount = 0;
                    this.dirtyDevices = {};
                    this.setSaveText(savingName, this.dirtyCount);
                    this.discardButton.disable();
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                }
            },
            this
        );
    },
    renameSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();

        if (node == null) {
            sel = this.apartmentTreePanel.getSelectionModel();
            node = sel.getSelectedNode();
        }

        if ((node == null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you want to rename"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId == null) ||
            (ids.groupId == null) || (ids.sceneNumber == null)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Could not determine scene from node"));
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var recId = ids.zoneId + '-' + ids.groupId + '-' + ids.sceneNumber;

        var record = sceneStore.getById(recId);

        // this should never be the case, however if by any chance we missed
        // something I prefer having a messagebox here instead of some
        // random error
        if (record === undefined) {
            record = new sceneStore.recordType({
                                                    name : "",
                                                    scene: ids.sceneNumber,
                                                    group: ids.groupId,
                                                    zone: ids.zoneId
                                                }, recId);
            sceneStore.add([record]);
        }

        Ext.Msg.prompt(dSS._("Rename scene"), dSS._("New name for scene"),
                function(btn, text) {
                if ((btn === 'ok') && (text !== record.get('name'))) {
                    Ext.Ajax.request({
                        url: '/json/zone/sceneSetName',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            id: ids.zoneId,
                            groupID: ids.groupId,
                            sceneNumber: ids.sceneNumber,
                            newName: text
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    var old = record.get('name');
                                    record.set('name', text);
                                    sceneStore.commitChanges();
                                    dSS.MessageBus.fireEvent('statusmessage',
                                        dSS.gt.strargs(dSS._("Scene %1 was renamed. New name: %2"),
                                                [dSS.util.getName(old), text]));
                                } else {
                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                                }
                            }
                            catch(err) {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene") + ': ' + err);
                            }
                        },
                        failure: function(result, request) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                        }
                    });
                }
            },
            this, false, record.get('name'));
    },

    getSelectedSceneList: function() {
        var node = this.getSelectedNode();
        if (!node || !node.isLeaf()) {
            // should never happen because the button is only shown when a
            // leaf node is selected
            return;
        }

        var sceneList = [];
        var scope = this;
        node.parentNode.eachChild(function(entry) {
            // don't draw separator nodes and also exclude currently selected
            // node
            if ((entry.isLeaf() !== true) || 
                (entry.attributes.isSeparator === true)) {
                return true;
            }
            var ids = scope.getZoneSceneGroup(entry.id);
            var item =
            [
                ids.sceneNumber,
                ids.zoneId,
                ids.groupId,
                entry.attributes.iconCls,
                entry.text
            ];
            sceneList.push(item);
            return true;
        });
        return sceneList;
    },

    saveToAnotherScene: function() {
        var win = new dSS.SaveSceneDialogWindow({
                sceneList: this.getSelectedSceneList(),
                caller: this,
                sceneToggleState: this.disableSceneFilter,
                height: 170,
                maskElement: this.findParentByType(Ext.Viewport).el
            });
        win.show(this);
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);

