Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputImpulseConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {

        this.device = config.device;

        this.loadedValue = undefined;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.selection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            value: 'retain',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [
                        'on',
                        dSS._("Turned on")
                    ],
                    [
                        'off',
                        dSS._("Turned off")
                    ],
                    [
                        'retain',
                        dSS._("Retain output")
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            border: false,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    height: 40,
                    layout: 'hbox',
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Output after an impulse called on " +
                                        "a single device") + ':',
                            width: 200
                        },
                        this.selection
                    ]
                }
            ]
        });

        dSS.devprops.OutputImpulseConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
            Ext.QuickTips.register({ target: this.selection.getEl(), 
                    text: dSS._("Valid when calling an impulse on a single device within a server app. Setting will not be used when calling an impulse from any preset.") });

            this.selection.on('select', function() {
                this.fireEvent('formchanged');
            },
            this);
        },
        this);

        this.on('beforedestroy', function() {
                this.closed = true;
                Ext.QuickTips.unregister(this.selection.getEl());
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.OUTPUT_AFTER_IMPULSE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.selection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedValue = data.json.result.output;
                this.selection.setValue(data.json.result.output);
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var value = this.selection.getValue();
            var dSUID = this.dSUID;
            var me = this;

            if (value === this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                    url: '/json/device/setOutputAfterImpulse',
                    params:
                    {
                        output: value,
                        dsuid: dSUID
                    },
                    failureMsg: dSS._("Could not save output after impulse settings"),
                    scope: me,
                    evt: 'formsubmitted'
                }
            );
        },
        this);
    }
});

Ext.reg('dssdevoutputimpulseconfigform', dSS.devprops.OutputImpulseConfigForm);

